function FindModel(Data, W, tr, StatNum, xoff, yoff, zoff, flBASE1, flBASE2, flBASE3, tp)
%function FindModel(Data, W, tr, StatNum, xoff, yoff, zoff, flBASE1, flBASE2, flBASE3, tp)
%	This function calculates the models for each USVG file in the
%	directory data
%
%	Data - Data corresponding to G
%	W - weights for each data point
%	tr - Number of Station for each event
%	StatNum - station number for each data point
%	tp - If equal to 1, find solution for event mean
%
% Example offsets vectors
%
%	xoff = 0:5:20;
%	yoff = 0:5:20;
%	zoff = 0:5:20;
%
% Example file base names
%
%	flBASE1 = 'data/50KM/dWG_50km_';
%	flBASE2 = 'data/50KM/USVG_50km_';
%	flBASE3 = 'data/50KM/MODS_50km';

nx = length(xoff);
ny = length(yoff);
nz = length(zoff);


% Intialize some variables

Gfile = [flBASE1 '0_0_0' '.dat'];
[tvd, tvW, tvG, GP] = ReadG(Gfile);
tvar = find(tr > 0);
nev = length(tvar);
nsta = max(StatNum);
noff = nx * ny * nz;

if tp == 1
	npar = GP.numx * GP.numy * GP.numz + nev + nsta;
	Folds = zeros(noff,npar-nev-nsta);
	StaticsDefined = 1;
else
	npar = GP.numx * GP.numy * GP.numz;
	Folds = zeros(noff,npar);
	StaticsDefined = 0;
end

Mods = zeros(noff,npar);
Ress = zeros(noff,npar);

XOF = zeros(1,noff);
YOF = zeros(1,noff);
ZOF = zeros(1,noff);

% Solve tomographic problem

%disp('Solving Tomographic Problem')
n = 0;
for k = 1:nz
	tic
	disp(['z offset: ' num2str(k)])
	for j = 1:ny
		for i = 1:nx
			clear G par tvd tvW GPdummy

			n = n + 1;
			flMID = [num2str(xoff(i)) '_' num2str(yoff(j)) '_' num2str(zoff(k))];
			USVfile = [flBASE2 flMID '.mat'];
			%disp(USVfile)

			par = load(USVfile);

			Gfile = [flBASE1 flMID '.dat'];
			%Gfile = [flBASE1 '0_0_0' '.dat'];
			[tvd, tvW, G, GPdummy] = ReadG(Gfile);
			par.G = G;

			%whos
			%figure(5)
			%pcolor(par.U);shading flat
			%pause
			%pcolor(par.V);shading flat
			%pause
			%plot(log10(diag(par.S)),'k--');
			nSing(n) = length(find(diag(par.S) > .2*par.S(1,1)));
			%disp(num2str(nSing))
			%pause
			%[M, R, Fold] = SolveInv(G.G, Data, W);
			[M, R, Fold] = SolveInvSVD(par.U, par.S, par.V, par.G, Data, W, nSing(n));
			Mods(n,:) = M';
			Folds(n,:) = Fold';
			Ress(n,:) = R';
			%if max(R) > 1
				%jj = find(R == max(R));
				%disp(['overstepped max R: ' USVGfile ' ind: ' num2str(jj)])
				%figure(3)
				%plot(R,'k--')
				%pause
			%end
			XOF(n) = xoff(i);
			YOF(n) = yoff(j);
			ZOF(n) = zoff(k);
		end
	end
toc
end

MODSfile = [flBASE3 '.mat'];
save(MODSfile,'Mods','Folds','Ress','XOF','YOF','ZOF','GP','nx','ny','nz',...
	'StaticsDefined','xoff','yoff','zoff','nev','nsta','Data','W')
disp(['Average nSing: ' num2str(mean(nSing))])
