function GI(W,tr,st,xoff,yoff,zoff,flBASE1,flBASE2)
%function GI(W,tr,st,xoff,yoff,zoff,flBASE1,flBASE2)
%
%	This function calculates U, S, and V for each G in the directory 
%	specfied by flBASE1 and outputs the results to flBASE2. Files in
%	flBASE1 were created with MakeG.csh
%
%	tr - number of traces for each event, used to solve for event static.
%	st - vector containing station number for each data point, used to
%	    solve for station static.
%
% Substitude 0 for the above parameters to indicate no use of that parameter.
%
%	W - weight given to each data point , if set to 1, all data
%	    will have the same weight.
%
% Define convention for offset bins, for example
%
%	xoff = 0:5:45;
%	yoff = 0:5:45;
%	zoff = 0:5:45;
%
% Input and output directories, for example
%
%	flBASE1 = 'data/50KM/dWG_50km_';
%	flBASE2 = 'data/50KM/USVG_50km_';
%

nx = length(xoff);
ny = length(xoff);
nz = length(xoff);

% Intialize some variables

file = [flBASE1 '0_0_0.dat'];
[tvd, tvw, G, GP] = ReadG(file);
nd = length(G(:,1));
nm = length(G(1,:));

if nd < nm
	n = nd;
else
	n = nm;
end

nev = length(find(tr > 0));
disp(['Number of Traces: ' num2str(nd)]);
disp(['Number of ModelParameters: ' num2str(nm)]);
if nev > 0
	disp(['Number of Events: ' num2str(nev)]);
else
	disp(['No solution for event mean']);
end
if length(st) == 1
	disp(['No solution for station static']);
end

% Solve tomographic problem
% First, generate eigenvalues and eigenvectors

for k = 1:nz
	for j = 1:ny
		for i = 1:nz
			clear tvd tvw G GP U S V s v
			flMID = [num2str(xoff(i)) '_' num2str(yoff(j)) '_' num2str(zoff(k))];
			file = [flBASE1 flMID '.dat'];
			disp(file)

			[tvd, tvw, G, GP] = ReadG(file);

			USVfile = [flBASE2 flMID '.mat'];
			disp(USVfile)

			if nev > 0
				disp('Adding Ev Static Solution to G')
				G = AddEv2G(G,tr);
			end
			if length(st) > 1
				disp('Adding Station Static Solution to G')
				G = AddSt2G(G,st);
			end

			disp('Solving for U, S, and V')
			[U, s, v] = svd(diag(W)*G,0);
			S = s(:,1:n);
			V = v(:,1:n);

			disp('Saving Data')
			save(USVfile,'U','S','V','W');
			disp('');
		end
	end
end


