#include <stdio.h>
#include <stdlib.h>
#include <stddef.h>
#include <math.h>
#include "bin.h"

/*
	This module produces distances in each bin
	given a ray path.
*/

double hyp(struct Pt *pt1, struct Pt *pt2)
{
	double Hypotenuse;
	double dx, dy, dz;

	dx = pt2->x - pt1->x;
	dy = pt2->y - pt1->y;
	dz = pt2->z - pt1->z;

	Hypotenuse = sqrt(dx * dx + dy * dy + dz * dz);

	return Hypotenuse;
}

int bin(struct Bin *bn, struct RayTrace *Ray)
{
	int iz, iy, ix, i;
	int dir;
	int orgPt, curPt;
	double dx, dy, dz;
	double totz, dist, dist1, bindist, *bndist;
	double slope1, intercept1, slope2, intercept2;
	float *x, *y, *z, *xup, *yup, *zup, X, Y, Z;
	float *xc, *yc, *zc, *xcup, *ycup, *zcup;
	float *xcR, *ycR, *zcR, *xccR, *yccR, *zccR, *xcRn, *ycRn, *zcRn;
	struct Pt FirstPt, PrevPt, NextPt, LastPt, dl, pt, pt0, pts[2];


	x = (float *)malloc((size_t) bn->numx * sizeof(float));
	y = (float *)malloc((size_t) bn->numy * sizeof(float));
	z = (float *)malloc((size_t) bn->numz * sizeof(float));
	xup = (float *)malloc((size_t) bn->numx * sizeof(float));
	yup = (float *)malloc((size_t) bn->numy * sizeof(float));
	zup = (float *)malloc((size_t) bn->numz * sizeof(float));
	xc = x;
	yc = y;
	zc = z;
	xcup = xup;
	ycup = yup;
	zcup = zup;
	for (ix = 0; ix < bn->numx; ++ix, ++xc, ++xcup)
	{
		*xc = bn->xo + ix * bn->dx;
		*xcup = *xc + bn->dx;
	}
	for (iy = 0; iy < bn->numy; ++iy, ++yc, ++ycup)
	{
		*yc = bn->yo + iy * bn->dy;
		*ycup = *yc + bn->dy;
	}
	for (iz = 0; iz < bn->numz; ++iz, ++zc, ++zcup)
	{
		*zc = bn->zo + iz * bn->dz;
		*zcup = *zc + bn->dz;
	}

	pt0.x = 0.0;
	pt0.y = 0.0;
	pt0.z = 0.0;

	totz = bn->zo + (bn->numz-1) * bn->dz;
	totz = z[bn->numz-1];
	xcR = Ray->x;
	ycR = Ray->y;
	zcR = Ray->z;
	FirstPt.x = *xcR;
	FirstPt.y = *ycR;
	FirstPt.z = *zcR;
	dist = 0.0;
	for (i = 0; i < Ray->pts; ++i)
	{
		NextPt.x = *xcR;
		NextPt.y = *ycR;
		NextPt.z = *zcR;
		dist1 = hyp(&FirstPt,&NextPt);	
		dist += dist1;	
		FirstPt.x = NextPt.x;
		FirstPt.y = NextPt.y;
		FirstPt.z = NextPt.z;
		++xcR; ++ycR; ++zcR;
	}

	bindist = 0.0;
	bndist = bn->dist;

//	BIN DATA

	zc = z;
	zcup = zup;
	for (iz = 0; iz < bn->numz; ++iz, ++zc, ++zcup)
	{
		yc = y;
		ycup = yup;
		for (iy = 0; iy < bn->numy; ++iy, ++yc, ++ycup)
		{
			xc = x;
			xcup = xup;
			for (ix = 0; ix < bn->numx; ++ix, ++xc, ++xcup, ++bndist)
			{
				*bndist = 0.0;
				xcR = Ray->x;
				ycR = Ray->y;
				zcR = Ray->z;
				xcRn = Ray->x + 1;
				ycRn = Ray->y + 1;
				zcRn = Ray->z + 1;
				orgPt = 0;

//				Check to see if bins exist between points
				while ((*zcR < *zc || *zcR >= *zcup ||
				        *ycR < *yc || *ycR >= *ycup || 
		     		        *xcR < *xc || *xcR >= *xcup) && 
				       (*zcRn < *zc || *zcRn >= *zcup ||
				        *ycRn < *yc || *ycRn >= *ycup || 
		     		        *xcRn < *xc || *xcRn >= *xcup))  
				{
					dl.x = *xcRn - *xcR;
					dl.y = *ycRn - *ycR;
					dl.z = *zcRn - *zcR;

					i = 0;

//					Check y-z planes
					if ((*xcR >= *xc && *xcRn < *xc) ||
					    (*xcR < *xc && *xcRn >= *xc))
					{
						slope1 = dl.z/dl.x;
						intercept1 = *zcR - slope1 * *xcR;
						slope2 = dl.y/dl.x;
						intercept2 = *ycR - slope2 * *xcR;
						Z = *xc * slope1 + intercept1; 
						Y = *xc * slope2 + intercept2; 
						if (Y >= *yc && Y < *ycup &&
						    Z >= *zc && Z < *zcup)
						{
							pts[i].x = *xc;
							pts[i].y = Y;
							pts[i].z = Z;
//							printf("1. %f %f %f\n",pts[i].x, pts[i].y, pts[i].z);
							i++;
						}
					}
					if ((*xcR >= *xcup && *xcRn < *xcup) ||
					    (*xcR < *xcup && *xcRn >= *xcup))
					{
						slope1 = dl.z/dl.x;
						intercept1 = *zcR - slope1 * *xcR;
						slope2 = dl.y/dl.x;
						intercept2 = *ycR - slope2 * *xcR;
						Z = *xcup * slope1 + intercept1; 
						Y = *xcup * slope2 + intercept2; 
						if (Y >= *yc && Y < *ycup &&
						    Z >= *zc && Z < *zcup)
						{
							pts[i].x = *xcup;
							pts[i].y = Y;
							pts[i].z = Z;
//							printf("2. %f %f %f\n",pts[i].x, pts[i].y, pts[i].z);
							i++;
						}
					}
	
//					Check x-z planes
					if ((*ycR >= *yc && *ycRn < *yc) ||
					    (*ycR < *yc && *ycRn >= *yc))
					{
						slope1 = dl.z/dl.y;
						intercept1 = *zcR - slope1 * *ycR;
						slope2 = dl.x/dl.y;
						intercept2 = *xcR - slope2 * *ycR;
						Z = *yc * slope1 + intercept1; 
						X = *yc * slope2 + intercept2; 
						if (X >= *xc && X < *xcup &&
						    Z >= *zc && Z < *zcup)
						{
							pts[i].x = X;
							pts[i].y = *yc;
							pts[i].z = Z;
//							printf("3. %f %f %f\n",pts[i].x, pts[i].y, pts[i].z);
							i++;
						}
					}
					if ((*ycR >= *ycup && *ycRn < *ycup) ||
					    (*ycR < *ycup && *ycRn >= *ycup))
					{
						slope1 = dl.z/dl.y;
						intercept1 = *zcR - slope1 * *ycR;
						slope2 = dl.x/dl.y;
						intercept2 = *xcR - slope2 * *ycR;
						Z = *ycup * slope1 + intercept1; 
						X = *ycup * slope2 + intercept2; 
						if (X >= *xc && X < *xcup &&
						    Z >= *zc && Z < *zcup)
						{
							pts[i].x = X;
							pts[i].y = *ycup;
							pts[i].z = Z;
//							printf("4. %f %f %f\n",pts[i].x, pts[i].y, pts[i].z);
							i++;
						}
					}
	
//					Check y-x planes
					if ((*zcR >= *zc && *zcRn < *zc) ||
					    (*zcR < *zc && *zcRn >= *zc))
					{
						slope1 = dl.x/dl.z;
						intercept1 = *xcR - slope1 * *zcR;
						slope2 = dl.y/dl.z;
						intercept2 = *ycR - slope2 * *zcR;
						X = *zc * slope1 + intercept1; 
						Y = *zc * slope2 + intercept2; 
						if (Y >= *yc && Y < *ycup &&
						    X >= *xc && X < *xcup)
						{
							pts[i].x = X;
							pts[i].y = Y;
							pts[i].z = *zc;
//							printf("5. %f %f %f\n",pts[i].x, pts[i].y, pts[i].z);
							i++;
						}
					}
					if ((*zcR >= *zcup && *zcRn < *zcup) ||
					    (*zcR < *zcup && *zcRn >= *zcup))
					{
						slope1 = dl.x/dl.z;
						intercept1 = *xcR - slope1 * *zcR;
						slope2 = dl.y/dl.z;
						intercept2 = *ycR - slope2 * *zcR;
						X = *zcup * slope1 + intercept1; 
						Y = *zcup * slope2 + intercept2; 
//						printf("x-y plane %f %f %f , %f %f\n",Ray->z[orgPt],Ray->z[orgPt+1],zup, X, Y);
						if (Y >= *yc && Y < *ycup &&
						    X >= *xc && X < *xcup)
						{
							pts[i].x = X;
							pts[i].y = Y;
							pts[i].z = *zcup;
//							printf("6. %f %f %f\n",pts[i].x, pts[i].y, pts[i].z);
							i++;
						}
					}

					if (i == 2)
					{
						*bndist = hyp(&pts[0], &pts[1]);
//						printf("INTER %d\n", i);
					}
							
					orgPt += 1;
					++xcR; ++ycR; ++zcR; ++xcRn; ++ycRn; ++zcRn;
					if (orgPt == Ray->pts-1)
						break;
				}
				

				if (!(*bndist))
				{
//				Check to see if point is inside bin
					xcR = Ray->x;
					ycR = Ray->y;
					zcR = Ray->z;
					orgPt = 0;
					while (!(*zcR >= *zc && *zcR < *zcup &&
					       *ycR >= *yc && *ycR < *ycup && 
			     		       *xcR >= *xc && *xcR < *xcup))  
					{
						++orgPt;
						++xcR; ++ycR; ++zcR;
						if (orgPt == Ray->pts)
							break;
					}
	
					if (orgPt < Ray->pts)
					{
						FirstPt.x = *xcR;
						FirstPt.y = *ycR;
						FirstPt.z = *zcR;
						for (dir = -1; dir < 2; dir += 2)
						{
							curPt = orgPt;
							xccR = xcR;
							yccR = ycR;
							zccR = zcR;
							while (*zccR >= *zc && *zccR < *zcup &&  
							       *yccR >= *yc && *yccR < *ycup && 
							       *xccR >= *xc && *xccR < *xcup) 
							{
								PrevPt.x = *xccR;
								PrevPt.y = *yccR;
								PrevPt.z = *zccR;
								curPt += dir;
								if (curPt < 0)
									break;
								xccR += dir;
								yccR += dir;
								zccR += dir;

								if (*zccR >= *zc && *zccR < *zcup &&
							            *yccR >= *yc && *yccR < *ycup &&
							            *xccR >= *xc && *xccR < *xcup) 
								{
									NextPt.x = *xccR;
									NextPt.y = *yccR;
									NextPt.z = *zccR;
									*bndist += hyp(&PrevPt, &NextPt);
								}
								else
								{
									LastPt.x = *xccR;
									LastPt.y = *yccR;
									LastPt.z = *zccR;
									if (LastPt.x > PrevPt.x)
									{
										dx = *xcup - PrevPt.x;
										dl.x = LastPt.x - PrevPt.x;
									}
									else
									{
										dx = PrevPt.x - *xc;
										dl.x = PrevPt.x - LastPt.x;
									}
									if (LastPt.y > PrevPt.y)
									{
										dy = *ycup - PrevPt.y;
										dl.y = LastPt.y - PrevPt.y;
									}
									else
									{
										dy = PrevPt.y - *yc;
										dl.y = PrevPt.y - LastPt.y;
									}
									if (LastPt.z > PrevPt.z)
									{
										dz = *zcup - PrevPt.z;
										dl.z = LastPt.z - PrevPt.z;
									}
									else
									{
										dz = PrevPt.z - *zc;
										dl.z = PrevPt.z - LastPt.z;
									}
									if (fabs(dz) > .00001 &&
									    fabs(dy) > .00001 &&
									    fabs(dx) > .00001)
									{
										if (dl.z/dl.x > dz/dx)  
										{
											if (dl.z/dl.y > dz/dy)
											{
												pt.x = dz * dl.x / dl.z;
												pt.y = dz * dl.y / dl.z;
												pt.z = dz;
											}
											else
											{
												pt.x = dy * dl.x / dl.y;
												pt.y = dy;
												pt.z = dy * dl.z / dl.y;
											}
										}
										else	
										{
											if (dl.y/dl.x > dy/dx)
											{
												pt.x = dy * dl.x / dl.y;
												pt.y = dy;
												pt.z = dy * dl.z / dl.y;
											}
											else
											{
												pt.x = dx;
												pt.y = dx * dl.y / dl.x;
												pt.z = dx * dl.z / dl.x;
											}
										}
										*bndist += hyp(&pt0, &pt);
									}
								}
							}
						}
					}
				}
				bindist += *bndist;
			}
		}
	}
	
	//printf("Dist: %f\t Distbin: %f\n",dist,bindist);
	free(x);
	free(y);
	free(z);
	free(xup);
	free(yup);
	free(zup);
}
