#include <stdio.h>
#include <stdlib.h>
#include <stddef.h>
#include <strings.h>
#include <math.h>
#include "../math/bin.h"
#include <time.h>

#define RPTSMAX 10000

/*
	This program produces distance measurements in each bin
	given a ray path and binning description

	to compile: gcc tomodWGmatrix.c bin.c -lm -o tomodWGmatrix

	bin.c requires the header file raytrace.h
*/

int main(int narg, char *argv[])
{
	int i, j, iz, iy, ix;
	int newfile;
	long int nd;
	double z;
	struct Bin bn;
	struct RayTrace Ray;
	char fname[100];	
	char vfname[100];	
	double *Vel;
	float df1, df2;
	float *Velocity;
	float *Depth;
	float Distance, data, weight;
	FILE *pfile;
	time_t t1, t2;

	//t1 = time(NULL);

	if (narg < 5)
	{
		fprintf(stderr,"\nUsage: tomodWGmatrix RayFile datavalue weight BinParamFile\n\n");
		fprintf(stderr,"\tThe RayFile must be of the form: x y z and be less than 10,000 pts\n");
		fprintf(stderr,"\tThe BinParamFile must be of the form:\n");
		fprintf(stderr,"\t\tInitialX XIncrement NumberXBins\n");
		fprintf(stderr,"\t\tInitialY YIncrement NumberYBins\n");
		fprintf(stderr,"\t\tInitialZ ZIncrement NumberZBins\n\n");
		return 1;
	}

	data = atof(argv[2]);
	weight = atof(argv[3]);

	nd = 0;
	if ((pfile = fopen("dWG.dat","r")) == NULL)
		newfile = 1;
	else
	{
		newfile = 0;
		fseek(pfile,0,SEEK_SET);
		fread(&nd,4,1,pfile);
	}
	fclose(pfile);
		
	strcpy(fname,argv[4]);
	if ((pfile = fopen(fname,"r")) == NULL)
	{
		fprintf(stderr,"BinParamFile, %s, could not be found.\n",fname);
		return 1;
	}
	else
	{
		fscanf(pfile,"%f %f %d", &df1, &df2, &bn.numx);
		bn.xo = df1;
		bn.dx = df2;
		fscanf(pfile,"%f %f %d", &df1, &df2, &bn.numy);
		bn.yo = df1;
		bn.dy = df2;
		fscanf(pfile,"%f %f %d", &df1, &df2, &bn.numz);
		bn.zo = df1;
		bn.dz = df2;
	}
	fclose(pfile);

	strcpy(fname,argv[1]);	
	if ((pfile = fopen(fname,"r")) == NULL)
	{
		fprintf(stderr,"RayFile, %s, could not be found.\n",fname);
		return 1;
	}
	else
	{
		i = 0;
		Ray.x = (float *)malloc((size_t) RPTSMAX*sizeof(float));
		Ray.y = (float *)malloc((size_t) RPTSMAX*sizeof(float));
		Ray.z = (float *)malloc((size_t) RPTSMAX*sizeof(float));
		while(fscanf(pfile,"%f %f %f", &Ray.x[i], &Ray.y[i], &Ray.z[i]) != EOF)
			++i;
		Ray.pts = i;
	}
	fclose(pfile);

	bn.dist = (double *)malloc((size_t) bn.numx * bn.numy * bn.numz * sizeof(double));
	bin(&bn, &Ray);

	free(Ray.x);
	free(Ray.y);
	free(Ray.z);

	++nd;
	if (newfile)
	{
		pfile = fopen("dWG.dat","wb");
		fwrite(&nd,4,1,pfile);
		fwrite(&bn.numx,4,1,pfile);
		fwrite(&bn.numy,4,1,pfile);
		fwrite(&bn.numz,4,1,pfile);
		df1 = (float)bn.xo;
		fwrite(&df1,4,1,pfile);
		df1 = (float)bn.yo;
		fwrite(&df1,4,1,pfile);
		df1 = (float)bn.zo;
		fwrite(&df1,4,1,pfile);
		df1 = (float)bn.dx;
		fwrite(&df1,4,1,pfile);
		df1 = (float)bn.dy;
		fwrite(&df1,4,1,pfile);
		df1 = (float)bn.dz;
		fwrite(&df1,4,1,pfile);
	}
	else
	{
		pfile = fopen("dWG.dat","r+b");
		fseek(pfile,0,SEEK_SET);
		fwrite(&nd,4,1,pfile);
	}
	fclose(pfile);

	pfile = fopen("dWG.dat","ab");
	fwrite(&data,4,1,pfile);
	fwrite(&weight,4,1,pfile);
	for (iz = 0; iz < bn.numz; ++iz)
		for (iy = 0; iy < bn.numy; ++iy)
			for (ix = 0; ix < bn.numx; ++ix)
			{
				Distance = (float)(bn.dist[ix + iy*bn.numx + iz*bn.numx*bn.numy]);
				fwrite(&Distance,4,1,pfile);
			}
	
	fclose(pfile);
	free(bn.dist);

	return 0;
}
